/*
 *  Copyright 2014  Andreas Cord-Landwehr <cordlandwehr@kde.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef NODETYPESTYLE_H
#define NODETYPESTYLE_H

#include "graphtheory_export.h"
#include "typenames.h"
#include <QObject>
#include <QSharedPointer>

namespace GraphTheory
{

class NodeTypeStylePrivate;

/**
 * \class Node
 */
class GRAPHTHEORY_EXPORT NodeTypeStyle : public QObject
{
    Q_OBJECT
public:
    NodeTypeStyle();
    ~NodeTypeStyle();
    void setColor(const QColor &color);
    QColor color() const;
    void setVisible(bool visible);
    bool isVisible() const;
    void setPropertyNamesVisible(bool visible);
    bool isPropertyNamesVisible() const;

Q_SIGNALS:
    void changed();
    void colorChanged(QColor);
    void visibilityChanged(bool);
    void propertyNamesVisibilityChanged(bool);

private:
    Q_DISABLE_COPY(NodeTypeStyle)
    const QScopedPointer<NodeTypeStylePrivate> d;
};
}

#endif
