/*
 *  Copyright 2013-2014  Andreas Cord-Landwehr <cordlandwehr@kde.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef CONSOLEMODULE_H
#define CONSOLEMODULE_H

#include "graphtheory_export.h"
#include "kernel/kernel.h"
#include <QObject>
#include <QList>
#include <QPair>

namespace GraphTheory
{
/**
 * \class ConsoleModule
 * This class provides an interface object for the scripting engine to write information to
 * the application console. The application widget that displays these information shall listen
 * to this object.
 */
class GRAPHTHEORY_EXPORT ConsoleModule : public QObject
{
    Q_OBJECT

public:
    explicit ConsoleModule(QObject *parent = 0);
    ~ConsoleModule();

    /**
     * Clear the backlog.
     */
    void clear();

    /**
     * Get all backlog since last clear.
     * \return the backlog
     */
    QList< QPair<GraphTheory::Kernel::MessageType, QString> > backlog() const;

public Q_SLOTS:
    /**
     * Print a log message \p message.
     */
    Q_INVOKABLE void log(const QString &message);

    /**
     * Print a debug message \p message.
     */
    Q_INVOKABLE void debug(const QString &message);

    /**
     * Print an error message \p message.
     */
    Q_INVOKABLE void error(const QString &message);

Q_SIGNALS:
    void message(const QString &message, GraphTheory::Kernel::MessageType type);

private:
    Q_DISABLE_COPY(ConsoleModule)
    QList< QPair<GraphTheory::Kernel::MessageType, QString> > m_backlog;
};
}

#endif
