/*
 *  Copyright 2013-2014  Andreas Cord-Landwehr <cordlandwehr@kde.org>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License or (at your option) version 3 or any later version
 *  accepted by the membership of KDE e.V. (or its successor approved
 *  by the membership of KDE e.V.), which shall act as a proxy
 *  defined in Section 14 of version 3 of the license.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SCRIPTAPIWIDGET_H
#define SCRIPTAPIWIDGET_H

#include "ui_scriptapiwidget.h"
#include <QWidget>

class QUrl;
class ScriptApiManager;
class ScriptApiModel;

/**
 * \class ScriptApiWidget
 * This Widget loads API information about all script interfaces and displays them.
 */
class ScriptApiWidget : public QWidget
{
    Q_OBJECT

public:
    /**
     * Default constructor.
     */
    explicit ScriptApiWidget(QWidget* parent);
    void showHtmlOutline(bool logHistory);

public Q_SLOTS:
    void showTreeOutline();
    void showHtmlOutline();
    void showDetails(const QModelIndex &index);
    void showObjectApi(const QString &id, bool logHistory);
    void showObjectApi(const QUrl &aliasPage);
    void historyGoBack();
    void historyGoForward();

private:
    ScriptApiManager *m_manager;
    ScriptApiModel *m_model;
    QUrl m_baseUrl;
    QList<QString> m_history;
    int m_historyPointer;
    Ui::ScriptApiWidget *ui;
};

#endif
